"""
    MoinMoin meta-data browser macro

    Displays meta data for easy browsing.

    Usage:

    [[BrowseMeta()]] Shows all meta data from all pages.
    [[BrowseMeta(search terms)]] Shows all meta data that match the search terms.

    @copyright: (c) 2005 by Charlie DeTar.
    @license: Gnu GPL (see COPYING for details)
"""
from MoinMoin import wikiutil, Page


class MetaBrowser:
    def __init__(self, macro, args):
        self.macro = macro

        # import required wiki plugins...
        MetaSearch = wikiutil.importWikiPlugin(macro.cfg, 'action', 'metasearch', 'MetaSearch')
        QueryParser = wikiutil.importWikiPlugin(macro.cfg, 'action', 'metasearch', 'MetaCapableQueryParser')
        MetaParser = wikiutil.importWikiPlugin(macro.cfg, 'parser', 'meta', 'MetaParser')
        if not MetaSearch or not MetaParser:
            return '''Error: metasearch.py action and meta.py parser must be
                    installed to use BrowseMeta macro.'''
        
        if macro.request.form.has_key('value'):
            args += " " + macro.form['value']
        case = int(macro.request.form.get('case', [0])[0])
        regex = int(macro.request.form.get('case', [0])[0])
        titlesearch = 0

        if args:
            parser = QueryParser(case=case, regex=regex, titlesearch=titlesearch)
            self.query = parser.parse_query(args)
        else:
            self.query = None

        self.meta = MetaSearch()
        self.parser = MetaParser('', self.macro.request)

    def getAllMetaData(self, macro):
        """ Searches over all wiki pages and extracts metadata. 

        If self.key is defined, returns metadata for that key.
        Otherwise, returns all metadata.
        """
        request = self.macro.request

        pages = request.rootpage.getPageList()

        fields = {}
        for name in pages:
            page = Page.Page(request, name)
            result = None
            if self.query:
                result = self.query.search(page)
            if result or not self.query:
                metadata = self.meta.matchesText(page)
                if metadata:
                    datalist = self.parser.parseMetadata(metadata)
                    for key,values in datalist:
                        fields[key] = fields.get(key, {})
                        for value in values:
                            fields[key][value] = fields[key].get(value, [])
                            fields[key][value].append(page.page_name)
        
        items = fields.items()
        items.sort()
        allmetadata = []
        for key, dict in items:
            valueitems = dict.items()
            valueitems.sort()
            allmetadata.append((key, valueitems))
        return allmetadata

    def formatBrowser(self, allmetadata):
        """ Displays the meta browser.  """
        f = self.macro.formatter 
        parser = self.parser

        out = ""
        for key, valuesAndPages in allmetadata:
            out += f.heading(1, 3)
            if key: 
                out += f.anchordef(key)
                out += key
            else:
                out += "(no key)"
            out += f.heading(0, 3)

            value, pages = valuesAndPages[0]
            if value or len(valuesAndPages) > 1:
                out += f.bullet_list(1)
                for value, pages in valuesAndPages:
                    if value:
                        out += f.listitem(1)
                        out += value #parser.buildLinks(f, (key, value), 'value')

                        for page in pages:
                            out += self.makePageLink(page)

                        out += f.listitem(0)
                out += f.bullet_list(0)
            else:
                pages = valuesAndPages[0][1]
                for page in pages:
                    out += self.makePageLink(page)

        out += f.bullet_list(0)

        return out

    def makePageLink(self, page):
        """ Make a small link to a standard wiki page, prepended by a separator. """
        f = self.macro.formatter
        out = ""
        out += f.small(1)
        out += f.text(", ")
        out += f.pagelink(1,page)
        out += f.text(page)
        out += f.pagelink(0)
        out += f.small(0)
        return out

def execute(macro, args):
    browser = MetaBrowser(macro, args)
    allmetadata = browser.getAllMetaData(macro)
    return browser.formatBrowser(allmetadata)
